const SDL = @import("sdl2");

const Game = @import("../Game.zig");
const Bag = @import("Bag.zig");
const Grid = @import("Grid.zig");
const Piece = @import("Piece.zig");

const color = @import("../color.zig");

const Self = @This();

renderer: *SDL.SDL_Renderer,
grid_cell_size: i32,
grid_pos_x: i32,
grid_pos_y: i32,

pub fn init(renderer: *SDL.SDL_Renderer) Self {
    // Calculate positions
    var aux_w: i32 = undefined;
    var aux_h: i32 = undefined;

    // TODO: have in mind the return value
    _ = SDL.SDL_GetRendererOutputSize(renderer, &aux_w, &aux_h);

    const grid_cell_size = @divFloor(@minimum(aux_w, aux_h), 32);
    const grid_pos_x = @divFloor(aux_w, 2) - (grid_cell_size * @divFloor(Grid.ncolumns, 2));
    const grid_pos_y = @divFloor(aux_h, 2) - (grid_cell_size * @divFloor(Grid.nrows + Grid.buffer, 2));
    return Self{
        .renderer = renderer,
        .grid_pos_x = grid_pos_x,
        .grid_pos_y = grid_pos_y,
        .grid_cell_size = grid_cell_size,
    };
}

pub fn renderBag(self: *Self, game: Game) void {
    const pos_x = self.grid_pos_x + ((Grid.ncolumns + 1) * self.grid_cell_size);
    const pos_y = self.grid_pos_y + (Grid.buffer * self.grid_cell_size);
    const cell_size = self.grid_cell_size;

    var rect = SDL.SDL_Rect{
        .x = pos_x,
        .y = pos_y,
        .w = cell_size,
        .h = cell_size,
    };

    //var trans: u8 = 0;
    var r: u8 = 0;
    var g: u8 = 0;
    var b: u8 = 0;
    var a: u8 = 255;

    for (game.bag.contents[0..5]) |_, i| {
        if (game.bag.contents[i]) |piece_type| {
            var piece = Piece.init(piece_type);

            for (piece.structure) |_, y| {
                for (piece.structure[y]) |_, x| {
                    if (piece.structure[y][x]) {
                        r = piece.color.r;
                        g = piece.color.g;
                        b = piece.color.b;
                        a = piece.color.a;
                    } else {
                        r = piece.color.r;
                        g = piece.color.g;
                        b = piece.color.b;
                        a = 50;
                    }

                    rect.x = pos_x + @intCast(i32, x) * cell_size;
                    rect.y = pos_y + (@intCast(i32, y) + @intCast(i32, i * piece.structure.len)) * cell_size;

                    _ = SDL.SDL_SetRenderDrawColor(self.renderer, r, g, b, a);
                    _ = SDL.SDL_RenderFillRect(self.renderer, &rect);
                    _ = SDL.SDL_SetRenderDrawColor(self.renderer, r -| 30, g -| 30, b -| 30, a);
                    _ = SDL.SDL_RenderDrawRect(self.renderer, &rect);
                }
            }
        }
    }
}

pub fn renderHeld(self: *Self, game: Game) void {
    const pos_x = self.grid_pos_x - (5 * self.grid_cell_size);
    const pos_y = self.grid_pos_y + (Grid.buffer * self.grid_cell_size);
    const cell_size = self.grid_cell_size;

    var rect = SDL.SDL_Rect{
        .x = pos_x,
        .y = pos_y,
        .w = cell_size,
        .h = cell_size,
    };

    //var trans: u8 = 0;
    var r: u8 = 0;
    var g: u8 = 0;
    var b: u8 = 0;
    var a: u8 = 255;

    if (game.held) |held_type| {
        var held = Piece.init(held_type);

        for (held.structure) |_, y| {
            for (held.structure[y]) |_, x| {
                if (held.structure[y][x]) {
                    r = held.color.r;
                    g = held.color.g;
                    b = held.color.b;
                    a = held.color.a;
                } else {
                    r = held.color.r;
                    g = held.color.g;
                    b = held.color.b;
                    a = 50;
                }

                rect.x = pos_x + @intCast(i32, x) * cell_size;
                rect.y = pos_y + @intCast(i32, y) * cell_size;

                _ = SDL.SDL_SetRenderDrawColor(self.renderer, r, g, b, a);
                _ = SDL.SDL_RenderFillRect(self.renderer, &rect);
                _ = SDL.SDL_SetRenderDrawColor(self.renderer, r -| 30, g -| 30, b -| 30, a);
                _ = SDL.SDL_RenderDrawRect(self.renderer, &rect);
            }
        }
    }
}

pub fn render(self: *Self, game: Game) void {
    game.grid.render(self.renderer, self.grid_cell_size, self.grid_pos_x, self.grid_pos_y);
    game.piece.render(self.renderer, self.grid_cell_size, self.grid_pos_x, self.grid_pos_y);
    game.shadow.render(self.renderer, self.grid_cell_size, self.grid_pos_x, self.grid_pos_y);
    renderBag(self, game);
    renderHeld(self, game);
}
